from abc import ABCMeta as _ABCMeta, abstractmethod as _abstractmethod
from functools import wraps as _wraps

def _uabstractmethod(f):
    @_wraps(f)
    def func(*args): raise NotImplementedError()
    return _abstractmethod(func)

class AbstractBasicCompressor(metaclass=_ABCMeta):
    @_uabstractmethod
    def compress(self, data): pass

    @_uabstractmethod
    def flush(): pass

class AbstractBasicDecompressor(metaclass=_ABCMeta):
    @_uabstractmethod
    def decompress(self, data): pass

    @property
    @_uabstractmethod
    def eof(self): pass

    @property
    @_uabstractmethod
    def unused_data(self): pass

class AbstractMemberInfo(metaclass=_ABCMeta):
    @property
    @_uabstractmethod
    def filename(self): pass

    @property
    @_uabstractmethod
    def size(self): pass

    @property
    @_uabstractmethod
    def mtime(self): pass

    # Yes, I am putting an implementation detail in an abstract class. No, I do
    # not care.
    def __str__(self):
        return '{}.{}({!r}, {}, {})'.format(self._module, self.__class__.__name__,
                                            self.filename, self.size, self.mtime)

    def __repr__(self): return str(self)

class AbstractAdvancedFile(metaclass=_ABCMeta):
    @_uabstractmethod
    def close(self): pass

    @_uabstractmethod
    def info_for(self, member): pass

    @_uabstractmethod
    def all_info(self): pass

    @_uabstractmethod
    def members(self): pass

    @_uabstractmethod
    def dump(self): pass

    @_uabstractmethod
    def add(self, path, arcname=None, *, recursive=True): pass

    @_uabstractmethod
    def add_data(self, path, data): pass

    @_uabstractmethod
    def extract(self, path=None, **kw): pass

    @_uabstractmethod
    def extract_all(self, path=None, members=None, **kw): pass

    @_uabstractmethod
    def open(self, member, universal_newlines=False, **kw): pass



from . import AbstractBasicCompressor, AbstractBasicDecompressor
from bz2 import BZ2Compressor as Compressor, BZ2Decompressor as Decompressor,\
                BZ2File as File, open

AbstractBasicCompressor.register(Compressor)
AbstractBasicDecompressor.register(Decompressor)



import io

class RFile:
    def __init__(self, name, fp):
        self.__name = name
        self.__fp = fp

    @property
    def name(self): return self.__name

    def read(self, size=-1):
        return self.__fp.read(size)

class RTextFile(io.TextIOWrapper):
    def __init__(self, name, *args, **kw):
        self.__name = name
        super(RTextFile, self).__init__(*args, **kw)

    @property
    def name(self): return self.__name



from util import *
from arclib import bz2
from bz2 import compress, decompress

def test_incremental_compress():
    basic_test_c(bz2.Compressor(), decompress)

def test_incremental_decompress():
    basic_test_d(bz2.Decompressor(), compress)



from . import AbstractAdvancedFile, AbstractMemberInfo
from zipfile import is_zipfile as test, ZipFile as _zfile, ZipInfo as _info
import os as _os, datetime as _datetime
from . import _rfile

class Info(AbstractMemberInfo):
    _module = 'zip'

    def __init__(self, info):
        self.info = info

    @property
    def filename(self): return self.info.filename
    @filename.setter
    def filename(self, value):
        self.info.filename = value

    @property
    def size(self): return self.info.file_size
    @size.setter
    def size(self, value):
        self.info.file_size = value

    @property
    def mtime(self): return _datetime.datetime(*self.info.date_time)
    @mtime.setter
    def mtime(self, value):
        self.info.date_time = tuple(value.timetuple()[:6])

class File(AbstractAdvancedFile):
    def __init__(self, zipfile):
        self.__zip = zipfile

    def __enter__(self):
        return self

    def __exit__(self, *_):
        self.close()

    def close(self):
        self.__zip.close()

    def info_for(self, member):
        return Info(self.__zip.getinfo(member))

    def all_info(self):
        return list(map(Info, self.__zip.infolist()))

    def members(self):
        return self.__zip.namelist()

    def dump(self):
        self.__zip.printdir()

    def add(self, path, arcname=None, recursive=True):
        if _os.path.isdir(path) and recursive:
            for root, dirs, files in _os.walk(path):
                subroot = root[len(path):]

                for fpath in files:
                    fullpath = _os.path.join(root, fpath)
                    destpath = fullpath[len(path):].lstrip('/')

                    if arcname:
                        destpath = _os.path.join(arcname, destpath)

                    self.__zip.write(fullpath, destpath)
        else:
            self.__zip.write(path, arcname)

    def add_data(self, path, data):
        self.__zip.writestr(path, data)

    def extract(self, member, path=None, *, pwd=None):
        self.__zip.extract(member, path, pwd)

    def extract_all(self, path=None, members=None, *, pwd=None):
        self.__zip.extractall(path, members, pwd=pwd)

    def open(self, member, universal_newlines=False, *, pwd=None):
        try:
            fp = self.__zip.open(member, pwd=pwd)
        except KeyError:
            return None

        if universal_newlines:
            return _rfile.RTextFile(member, fp)
        else:
            return _rfile.RFile(member, fp)

def open(*args, **kw):
    return File(_zfile(*args, **kw))

def openobj(fileobj, **kw):
    return File(_zfile(fileobj, **kw))



from util import *
from arclib import lzma
from lzma import compress, decompress

def test_incremental_compress():
    basic_test_c(lzma.Compressor(), decompress)

def test_incremental_decompress():
    basic_test_d(lzma.Decompressor(), compress)



from . import AbstractAdvancedFile, AbstractMemberInfo
from tarfile import is_tarfile as test, TarFile as _tfile, TarInfo as _info
import datetime as _datetime, io as _io, time as _time
from . import _rfile

class Info(AbstractMemberInfo):
    _module = 'tar'

    def __init__(self, info, **kw):
        self.info = info

    @property
    def filename(self): return self.info.name
    @filename.setter
    def filename(self, value):
        self.info.name = value

    @property
    def size(self): return self.info.size
    @size.setter
    def size(self, value):
        self.info.size = value

    @property
    def mtime(self): return _datetime.datetime.fromtimestamp(self.info.mtime)
    @mtime.setter
    def mtime(self, value):
        self.info.mtime = value.timestamp()

class File(AbstractAdvancedFile):
    def __init__(self, tar):
        self.__tar = tar

    def __enter__(self):
        return self

    def __exit__(self, *_):
        self.close()

    def close(self):
        self.__tar.close()

    def info_for(self, member):
        return Info(self.__tar.getmember(member))

    def all_info(self):
        return list(map(Info, self.__tar.getmembers()))

    def members(self):
        return self.__tar.getnames()

    def dump(self):
        self.__tar.list()

    def add(self, path, arcname=None, *, recursive=True):
        self.__tar.add(path, arcname, recursive=recursive)

    def add_data(self, path, data):
        if isinstance(path, Info):
            info = path.info
        else:
            info = _info(path)
            info.size = len(data)
            info.mtime = _time.time()

        self.__tar.addfile(info, _io.BytesIO(data))

    def extract(self, member, path=None):
        self.__tar.extract(member, '' if path is None else path)

    def extract_all(self, path=None, members=None):
        self.__tar.extractall('.' if path is None else path, members)

    def open(self, member, universal_newlines=False):
        fp = self.__tar.extractfile(member)
        if fp is None:
            return None

        if universal_newlines:
            return _rfile.RTextFile(member, fp)
        else:
            return _rfile.RFile(member, fp)

def open(*args, **kw):
    return File(_tfile.open(*args, **kw))

def openobj(fileobj, **kw):
    return File(_tfile.open(name=kw.pop('name', None), fileobj=fileobj, **kw))



import sys, os, string
sys.path.insert(0, os.path.abspath('.'))
letters = bytes(string.ascii_letters, 'ascii')

def basic_test_c(c, d):
    res = b''
    for b in letters:
        res += c.compress(bytes([b]))
    res += c.flush()
    assert d(res) == letters

def basic_test_d(d, c):
    res = b''
    for b in c(bytes(letters)):
        res += d.decompress(bytes([b]))
    assert res == letters
    assert not d.unused_data



from setuptools import setup

with open('README.rst') as f:
    descr = f.read()

setup(name='arclib',
      version='0.2',
      description="A unified API for accessing Python's compression formats",
      long_description=descr,
      author='Ryan Gonzalez',
      author_email='rymg19@gmail.com',
      url='https://github.com/kirbyfan64/arclib',
      packages=['arclib'],
      classifiers=[
          'Programming Language :: Python :: 3 :: Only',
          'Topic :: System :: Archiving',
          'License :: OSI Approved :: MIT License',
      ],
)



from . import AbstractBasicCompressor, AbstractBasicDecompressor
from lzma import LZMACompressor as Compressor, LZMADecompressor as Decompressor,\
                 LZMAFile as File, open

AbstractBasicCompressor.register(Compressor)
AbstractBasicDecompressor.register(Decompressor)
